/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "definice.h"
#include "vyjimka.h"
#include "syntaxstrom.h"

Definice::Definice (const QString & j) {
   jmeno =j;
   zabudovana = false;
   cachePlatna = false;
   _jeFce = false;
}

QString Definice::echo (){
   QString str=QString("<SPAN CLASS=\"%1\">").arg((jeFce())?("DefiniceFce"):("Definice"));
   if (cachePlatna && (!jeFce())) str+="<SPAN CLASS=\"DefinicePlatna\">";
   str+= jmeno;
   if (cachePlatna && (!jeFce())) str+="</SPAN>";
   str+="</SPAN>";
   str+=spolecneEcho();
   return str;

}

bool Definice::jeDefinovana(){
   return (vyraz!=0) || zabudovana;
}

double Definice::eval(ParametryFce * par){
   if (cachePlatna && !_jeFce) return cacheHodnota;
   if (!jeDefinovana()) throw VyjimkaTextRadek("Identifikátor "+jmeno + " není definován",cisloRadku);
   try {
      cacheHodnota =  vyraz->eval(par);
   }
   catch (VyjimkaText v){
      throw VyjimkaTextRadek(v.text,cisloRadku);
   }
   cachePlatna = true;
   return cacheHodnota;
}

void Definice::zneplatni(){
   if (!cachePlatna) return;
   cachePlatna=false;
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavislostiShora.begin(); tmp!= zavislostiShora.end(); tmp++)
      (*tmp)->zneplatni();


}

void Definice::nahlasZavislosti(){
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiNa.begin(); tmp!= zavisiNa.end(); tmp++)
      (*tmp)->prijmiZavislost(this);
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiPresIf.begin(); tmp!= zavisiPresIf.end(); tmp++)
      (*tmp)->prijmiZavislost(this);
}

void Definice::odhlasZavislosti(){
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiNa.begin(); tmp!= zavisiNa.end(); tmp++)
      (*tmp)->zrusZavislost(this);
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiPresIf.begin(); tmp!= zavisiPresIf.end(); tmp++)
      (*tmp)->zrusZavislost(this);
}

void Definice::prijmiZavislost(Definice *a){
   zavislostiShora.insert(a);
}

void Definice::zrusZavislost(Definice *a){
   zavislostiShora.erase(a);
}
