/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DEFINICE_H_INCLUDED
#define DEFINICE_H_INCLUDED

#include "definice_baze.h"
#include "parametry_fce.h"

/**
 * Objekt pro uchovani definice promenne/funkce. Pracuje s cache a spravuje zavislosti shora.
 * @author Martin Plicka
 */
class Definice: public DefiniceBaze {
   public:

      /**
       * Jmeno identifikatoru.
       */
      QString jmeno;

      /**
       * Hodnota vypoctena v ramci fce eval().
       */
      double cacheHodnota;

      /**
       * True, pokud lze pri vypoctu pouzit cache.
       */
      bool cachePlatna;

      /**
       * True, pokud je funkce zabudovana, tedy nelze ji predefinovat.
       * Promenne se pak chovaji jako konstanty.
       */
      bool zabudovana;

      /**
       * Mnozina definic, jejichz hodnoty zavisi na hodnote teto definice.
       */
      SadaDefinic zavislostiShora;

      /**
       * Kontruktor.
       * @param jmeno Jmeno identifikatoru.
       */
      Definice (const QString & jmeno);

      virtual ~Definice(){};

      /**
       * Vypise definici funkce/promenne.
       * @return Retezec s vystupem.
       */
      virtual QString echo();

      /**
       * Zkontroluje, zdali ma promenna/funkce prirazenu definici.
       * Promenna/funkce totiz nemusi byt definovana, protoze ji staci pouzit v nejakem vzorci.
       * @return True, pokud je promenna definovana.
       */
      bool jeDefinovana();

      /**
       * Vyhodnoti funkci/promennou. Pripadne pouzije hodnoty v cache.
       * @param parametry Parametry funkce.
       * @return navratova hodnota
       */
      virtual double eval(ParametryFce * parametry);

      /**
       * Zneplatni cache
       */
      void zneplatni();

      /**
       * oznami definicim, ze na nich zavisi
       */
      void nahlasZavislosti();

      /**
       * Zrusi ohlaseni zavislosti (napr. pri predefinovani existujici promenne
       */
      void odhlasZavislosti();

      /**
       * prijmi oznameni, ze na definici nekdo zavisi
       * @param d odkaz na zavisejici definici
       */
      void prijmiZavislost(Definice * d);

      /**
       * prijmi zruseni zavislosti
       * @param d odkaz na jiz nezavisejici definici
       */
      void zrusZavislost(Definice * d);
};


#endif // DEFINICE_H_INCLUDED
