/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "definice_baze.h"
#include "prekladac.h"
#include "vyjimka.h"

DefiniceBaze::DefiniceBaze(){
   cisloRadku = Prekladac::log.cisloRadku;
   vyraz = 0;
}

bool DefiniceBaze::pridejParametr (const QString & s){
   _jeFce = true;
   if (find (parametry.begin(), parametry.end(), s)!= parametry.end()) return false;
   parametry.push_back(s);
   return true;
}

QString & DefiniceBaze::operator[] (int i) {
   return parametry[i];
}

void DefiniceBaze::smazParametry() { //smaze seznam parametru
   parametry.clear();
}

int DefiniceBaze::indexParametru(const QString & s){ // vrati index parametru ve vectoru
   for (unsigned i=0; i<parametry.size(); i++) if (parametry[i]==s) return i;
   return -1;
}

int DefiniceBaze::pocetParametru() {
   return parametry.size();
}

bool DefiniceBaze::jeFce(){
   return _jeFce;
}

void DefiniceBaze::jeFce(bool b) {
   _jeFce = b;
   if (!b && parametry.size()>0) smazParametry();
}

void DefiniceBaze::nastav(const DefiniceBaze & b){


   cisloRadku = b.cisloRadku;
   parametry = b.parametry; //seznam parametru funkce
   _jeFce = b._jeFce;
   vyraz = b.vyraz;
   zavisiNa = b.zavisiNa;
   zavisiPresIf= b.zavisiPresIf;


}

QString DefiniceBaze::spolecneEcho(){
   QString str="";
   if (jeFce()){
      str+="<SPAN CLASS=\"ZavorkaDefinice\">(</SPAN>";
      bool a = true;

      for (vector<QString>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
         if (a) a=false;
         else str+=",";
         str+= QString("<SPAN CLASS=\"DefiniceParametr\">")+(*tmp)+"</SPAN>";
      }
      str+=QString("<SPAN CLASS=\"ZavorkaDefinice\">)</SPAN>");
   }
   str+=QString(" <SPAN CLASS=\"Prirazeni\">=</SPAN> ");
   if (vyraz) str+=vyraz->echo();
   else str+=QString("<SPAN CLASS=\"Chyba\">nedef.</SPAN>");
   //str +=echoZavislosti();
   return str;

}

void DefiniceBaze::smazZavislosti() {
   zavisiNa.clear();
   zavisiPresIf.clear();
}

void DefiniceBaze::smazZavislost(Definice*a){
   set<Definice*,PorovnejDefinice>::iterator tmp;
   /*tmp = zavisiNa.find(a);
   if (tmp != (zavisiNa.end()))*/
   zavisiNa.erase(a);
   /*tmp = zavisiPresIf.find(a);
   if (tmp != (zavisiPresIf.end()))*/
   zavisiPresIf.erase(a);
}

void DefiniceBaze::zjistiZavislosti(){
   smazZavislosti();
   vyraz->zjistiZavislosti(&zavisiNa, &zavisiPresIf, false);
}

QString DefiniceBaze::echoZavislosti(){
   QString str = "|";
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiNa.begin(); tmp!= zavisiNa.end(); tmp++)
      str+=(*tmp)->jmeno+" ";
   str+= "|";
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiPresIf.begin(); tmp!= zavisiPresIf.end(); tmp++)
      str+=(*tmp)->jmeno+" ";
   return (str);
}

bool DefiniceBaze::zkontrolujRekurzi(Definice * a, bool vIf){
//    QMessageBox::information(hOkno, "Kontrola Rekurze", "f");
   bool presIf=false;
   //prime zavislosti
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiNa.begin(); tmp!= zavisiNa.end(); tmp++){
      if (!*tmp) continue;
      if (!vIf) {
         // pokud nejsme vnoreni v if, konrolujeme dusledne vsechno na vyskyt rekurze
         if ((*tmp)==a) throw Vyjimka();
         presIf = (*tmp)->zkontrolujRekurzi(a,vIf) || presIf;
      }
      else {
         // pokud jsme v if, tak kontrolujeme pouze pokud jsme nic nenasli, jinak hned skoncime
         presIf = presIf || ((*tmp)==a);
         if (!presIf) presIf = (*tmp)->zkontrolujRekurzi(a,vIf);
         else break;
      }

   }

   //zavislosti pres IF
   for (set<Definice*,PorovnejDefinice>::iterator tmp = zavisiPresIf.begin(); tmp!= zavisiPresIf.end(); tmp++){
      //zavislost pres if, nastavime true
      if ((*tmp)==a) presIf = true;
      // zde uz muze byt jen zavislost pres if, takze ma prednost dosavadni vysledek
      presIf = presIf || (*tmp)->zkontrolujRekurzi(a,true);
   }
   return presIf;
}
