/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "hlavniokno.h"
#include <QMessageBox>
#include <QCloseEvent>
#include <QTextStream>
#include "prekladac.h"
#include "about.h"

HlavniOkno::HlavniOkno(QWidget *parent) :QMainWindow(parent){
   setupUi(this);
   connect (this->actionKonec, SIGNAL(activated()), this, SLOT(close()));
   connect (this->actionO, SIGNAL(activated()), this, SLOT(oProgramu()));
   connect (this->pushButtonOdeslat, SIGNAL(clicked()), this, SLOT(odeslat()));
   connect (this->lineEditPrikaz, SIGNAL(returnPressed ()), this, SLOT(odeslat()));
   connect (this->actionOtevrit, SIGNAL(activated ()), this, SLOT(nacist()));
   connect (this->actionUp, SIGNAL(activated ()), this, SLOT(nahoru()));
   connect (this->actionDown, SIGNAL(activated ()), this, SLOT(dolu()));
   for (int i = 0; i<10; i++) prikazy[i] = "";
   kurzor=0;
   aktualni = 0;
}

void HlavniOkno::odeslat(){
   if (lineEditPrikaz->text() != "" ) {
      prikazy[(aktualni++) % 10] = lineEditPrikaz->text();
      prikazy[(aktualni) %10] = "";
   }
   kurzor = aktualni % 10;
   Prekladac::preloz(lineEditPrikaz->text());
   obnovitLog();
   lineEditPrikaz->clear();
}


void HlavniOkno::oProgramu(){
   (AboutBox::getInstance())->exec();
}

void HlavniOkno::closeEvent ( QCloseEvent * event ){
   if (QMessageBox::Yes == QMessageBox::question ( this, "Opravdu?", "Chcete program opravdu ukončit ?", QMessageBox::Yes| QMessageBox::No, QMessageBox::No)) qApp->quit();
   else event->ignore();

}

void HlavniOkno::nacist(){
   QString soubor;
   QString tmp;
   if ((soubor= QFileDialog::getOpenFileName(this, "", "./","Soubory MiniMath (*.*)"))!=NULL) {
      QFile data(soubor);
      if (data.open(QFile::ReadOnly)) {
         QTextStream in(&data);
         while (!in.atEnd()){
            Prekladac::preloz(tmp = in.readLine());
            if (!tmp.isEmpty() ) {
               prikazy[(aktualni++) % 10] = tmp;
               prikazy[(aktualni) %10] = "";
            }

         }
         data.close();
      }
      obnovitLog();
      kurzor = aktualni % 10;
   }
   lineEditPrikaz->setFocus();
}

void HlavniOkno::obnovitLog(){
   textEditLog->setText(Prekladac::log.echo());
   textEditLog->moveCursor(QTextCursor::End);
}

void HlavniOkno::nahoru(){
   if (prikazy[(kurzor+9) % 10] !="") lineEditPrikaz->setText(prikazy[(kurzor+=9) % 10]);
}

void HlavniOkno::dolu(){
   if (prikazy[(kurzor) % 10] !="" ) lineEditPrikaz->setText(prikazy[(++kurzor) % 10]);
}
