/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "lexan.h"
#include "vyjimka.h"
#include "strtools.h"

using namespace std;

namespace LexAn {

const char * Identifikace[29] = {"nic","identifikátor","číslo","řetězec","+","-","*","/","^","&amp;","|","!","(",")","&gt;","&lt;",
                                 "&ge;","&le;","==","&ne;","=",",","konec řádku","read","write","if","reset","def","defint"
                                };


/**
 * Buffer pro vstup
 */
QString vstup="";

/**
 * Aktualni pozice v bufferu
 */
int poziceVstup=0;

/**
 * Pozice posledniho nalezeneho lexikalniho elementu
 */
int posledniElement=0;

/**
 * reprezentuje typ vstupniho symbolu
 */
char symbol;

/**
 * reprezentuje skutecne nacteny znak
 */
QChar znak=' ';

/**
 * Naposledy precteny lexikalni element
 */
LexTyp lexElement;

/**
 * Naposledy precteny retezec
 */
QString lexRetez;

/**
 * Naposledy prectene cislo
 */
double lexHodnota;

/**
 * Nacte dalsi znak ze vstupniho bufferu
 */
QChar cti(){
   if (poziceVstup>=vstup.length()) return QChar('\n');
   return vstup[poziceVstup++];
};

/**
 * Precte ze vstupniho bufferu znak, a identifikuje jej do prislusnych promennych
 */
void ctiSymbol(){
   if (symbol!=0) znak = cti(); //bude jine, az bude v hl. programu
   if (znak.isLetter()) {
      symbol = 'p';
      return;
   }
   if (znak.isDigit()) {
      symbol = 'c';
      return;
   }
   if (znak=='\n') {
      symbol = 'e';
      return;
   }
   if (!znak.isPrint()) {
      symbol = 'x';
      return;
   }
   symbol=znak.toAscii();
};

/**
 * Vraci symbol zpet na vstup
 */
inline void VratSymbol(){
   symbol=0;
};

/**
 * Precte ze vstupniho bufferu lexikalni element
 */
void ctiElement(){
   posledniElement=poziceVstup;
   ctiSymbol();
   lexElement = LexNic ;
   lexRetez = "";
   lexHodnota =0;
   while (symbol==' ') ctiSymbol(); //vynecha mezery na zacatku
   QString str=""; //sem shromazdujeme vstupni znaky pro identifikatory, cislo, retezce

   switch (symbol){
      case 'p':
         str=znak;
         goto Ident;
      case 'c':
         str=znak;
         goto Cislo;
      case '.':
         str=".";
         goto CisloTecka;
      case '#':
         goto Komentar;
      case '"':
         goto Retez;
      case '=':
         goto Prirazeni;
      case '!':
         goto Not;
      case '<':
         goto Mensi;
      case '>':
         goto Vetsi;
      case '+':
         lexElement=LexPlus;
         return;
      case '-':
         lexElement=LexMinus;
         return;
      case '*':
         lexElement=LexKrat;
         return;
      case '/':
         lexElement=LexDeleno;
         return;
      case '^':
         lexElement=LexMocnina;
         return;
      case '&':
         lexElement=LexAnd;
         return;
      case '|':
         lexElement=LexOr;
         return;
      case '(':
         lexElement=LexLZavorka;
         return;
      case ')':
         lexElement=LexPZavorka;
         return;
      case 'e':
         lexElement=LexEnter;
         return;
      case ',':
         lexElement=LexCarka;
         return;
      default:
         throw VyjimkaText(QString("Nerozpoznaný symbol <B>\"")+str+znak+"\"</B>");
         return;
   }
Ident:
   ctiSymbol();
   switch (symbol) {
      case 'p':
      case 'c':
         str+=znak;
         goto Ident;
      default:
         if (str=="write") {
            lexElement=LexWrite;
            VratSymbol();
            return;
         }
         if (str=="read") {
            lexElement=LexRead;
            VratSymbol();
            return;
         }
         if (str=="if") {
            lexElement=LexIf;
            VratSymbol();
            return;
         }
         if (str=="reset") {
            lexElement=LexReset;
            VratSymbol();
            return;
         }
         if (str=="def") {
            lexElement=LexDef;
            VratSymbol();
            return;
         }
         if (str=="defint") {
            lexElement=LexDefInt;
            VratSymbol();
            return;
         }
         lexElement=LexIdentifikator;
         lexRetez=str;
         VratSymbol();
         return;
   }
Cislo:
   ctiSymbol();
   switch (symbol) {
      case 'c':
         str+=znak;
         goto Cislo;
      case '.':
         str+='.';
         goto CisloTecka;
      case 'p':
         if (znak=='e' || znak=='E') {
            str+='E';
            goto CisloExponent;
         }
      default:
         lexElement=LexCislo;
         lexHodnota=str.toDouble();
         VratSymbol();
         return;
   }
CisloTecka:
   ctiSymbol();
   switch (symbol){
      case 'c':
         str+=znak;
         goto CisloDeset;
      default:
         throw VyjimkaText(QString("Nerozpoznaný symbol <B>\"")+str+znak+"\"</B>");
         return;
   }
CisloDeset:
   ctiSymbol();
   switch (symbol){
      case 'c':
         str+=znak;
         goto CisloDeset;
      case 'p':
         if (znak=='e' || znak=='E') {
            str+='E';
            goto CisloExponent;
         }
      default:
         lexElement=LexCislo;
         lexHodnota=str.toDouble();
         VratSymbol();
         return;
   }
CisloExponent:
   ctiSymbol();
   switch (symbol){
      case '-':
         str+='-';
         goto CisloExponentMinus;
      case 'c':
         str+=znak;
         goto CisloExponent2;
      default:
         throw VyjimkaText(QString("Nerozpoznaný symbol <B>\"")+str+znak+"\"</B>");
         return;
   }
CisloExponentMinus:
   ctiSymbol();
   switch (symbol){
      case 'c':
         str+=znak;
         goto CisloExponent2;
      default:
         throw VyjimkaText(QString("Nerozpoznaný symbol <B>\"")+str+znak+"\"</B>");
         return;
   }
CisloExponent2:
   ctiSymbol();
   switch (symbol){
      case 'c':
         str+=znak;
         goto CisloExponent2;
      default:
         lexElement=LexCislo;
         lexHodnota=str.toDouble();
         VratSymbol();
         return;
   }
Retez:
   ctiSymbol();
   switch (symbol){
      case '"':
         lexElement=LexRetez;
         lexRetez=str;
         return;
      case '\\':
         goto Retez2; // umoznime vlozit uvozovky a zpetne lomitko
      case 'x':
         throw VyjimkaText("Řetězec obsahuje nepovolené znaky");
         return; //netisknutelne+enter neberem
      case 'e':
         throw VyjimkaText("Očekávám <B>\"</B> místo konce řádku");
         return;
      default:
         str+=znak;
         goto Retez;
   }
Retez2:
   ctiSymbol();
   switch (symbol){
      case 'x':
         throw VyjimkaText("Řetězec obsahuje nepovolené znaky");
         return; //netisknutelne+enter neberem
      case 'e':
         throw VyjimkaText("Očekávám <B>\"</B> místo konce řádku");
         return;
      default:
         str+=znak;
         goto Retez;
   }
Prirazeni:
   ctiSymbol();
   switch (symbol){
      case '=':
         lexElement=LexRovno;
         return;
      default:
         lexElement=LexPrirazeni;
         VratSymbol();
         return;
   }
Not:
   ctiSymbol();
   switch (symbol){
      case '=':
         lexElement=LexNerovno;
         return;
      default:
         lexElement=LexNot;
         VratSymbol();
         return;
   }
Mensi:
   ctiSymbol();
   switch (symbol){
      case '=':
         lexElement=LexMensiRovno;
         return;
      default:
         lexElement=LexMensi;
         VratSymbol();
         return;
   }
Vetsi:
   ctiSymbol();
   switch (symbol){
      case '=':
         lexElement=LexVetsiRovno;
         return;
      default:
         lexElement=LexVetsi;
         VratSymbol();
         return;
   }
Komentar:
   ctiSymbol();
   while (symbol!='e') {
      str += znak;
      ctiSymbol();
   }
   lexElement = LexEnter;
   lexRetez = str;
   return;
}

void init(const QString & s){
   vstup = s;
   symbol='e'; //cokoliv nenuloveho, musime lexan prinutit cist
   poziceVstup = 0;
   posledniElement=0;
   lexElement=LexNic;
}


LexTyp ukaz(){
   return (lexElement == LexNic)?(ctiElement(), lexElement):(lexElement);
}

bool srovnej(const LexTyp & l){
   if (ukaz()==l) {
      lexElement = LexNic;
      return true;
   }
   else {
      throw VyjimkaText(QString("Očekávám <B>\"")+Identifikace[l]+"\"</B> místo <B>\""+Identifikace[lexElement]+ "\"</B>, znak "+Stringify(posledniElement+1));
      return false;
   }
}


}
