/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LEXAN_H
#define __LEXAN_H

#include <qstring.h>

/**
 * Lexikalni analyzator.
 * Obsahuje veskere funkce lexikalniho analyzatoru
 * @author Martin Plicka
 */
namespace LexAn {

/**
 * Jmena jednotlivych syntaktickych elementu
 */
extern const char * Identifikace[];

/**
 * Typy lexikalnich elementu
 */
typedef enum LexTyp {
   /*0*/    LexNic,
   /*1*/    LexIdentifikator,
   /*2*/    LexCislo,
   /*3*/    LexRetez,
   /*4*/    LexPlus,        // +
   /*5*/    LexMinus,       // -
   /*6*/    LexKrat,        // *
   /*7*/    LexDeleno,      // /
   /*8*/    LexMocnina,     // ^
   /*9*/    LexAnd,         // &
   /*10*/    LexOr,          // |
   /*11*/    LexNot,         // !
   /*12*/    LexLZavorka,    // (
   /*13*/    LexPZavorka,    // )
   /*14*/    LexVetsi,       // >
   /*15*/    LexMensi,       // <
   /*16*/    LexVetsiRovno,  // >=
   /*17*/    LexMensiRovno,  // <=
   /*18*/    LexRovno,       // ==
   /*19*/    LexNerovno,     // !=
   /*20*/    LexPrirazeni,   // =
   /*21*/    LexCarka,       // ,
   /*22*/    LexEnter,       // \n
   /*23*/    LexRead,       // funkce read(), ma jinou syntaxi nez bezne funkce, proto je klicovym slovem
   /*24*/    LexWrite,      //funkce write() -- // --
   /*25*/    LexIf,         //podmineny vyraz if(cond, ifTrue, ifFalse)
   /*26*/    LexReset,     //prikaz reset, restart prostredi, rezervovano pro budoucnost
   /*27*/    LexDef,       // vypis definic
   /*28*/    LexDefInt,    // vypis zabudovanych funkci a konstant
};

/**
 * Naposledy precteny lexikalni element
 */
extern LexTyp lexElement;

/**
 * Naposledy precteny retezec
 */
extern QString lexRetez;

/**
 * Naposledy prectene cislo
 */
extern double lexHodnota;

/**
 * Precte dalsi element ze vstupu
 */
void ctiElement();

/**
 * Inicializuje vstupni buffer radkem kodu
 */
void init(const QString & s);

/**
 * Ukaze prvni dostupny symbol na vstupu
 */
LexTyp ukaz();

/**
 * Odebere dany lexikalni element ze vstupu
 * @param l typ daneho elementu ke srovnani
 * @return true, pokud srovnani probehlo dobre
 * @throw VyjimkaText pokud je spatne srovnani, tj. na vstupu je neco jineho, nez je ocekavano
 */
bool srovnej (const LexTyp & l);
};
#endif
