#include "log.h"
/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "format.h"
#include "strtools.h"
#include "prekladac.h"
#include "hlavniokno.h"
#include <qurl.h>

extern QString Adresar;

LogRadek::LogRadek(const QString & chyba) {
   typ = LogTypRadek;
   chyboveHlaseni=chyba;
}

QString LogRadek::echo(){
   if (!chyboveHlaseni.isEmpty()) return QString("<BR>&nbsp;&nbsp;<SPAN CLASS=\"Chyba\">- - ")+chyboveHlaseni+"</SPAN>";
   else return "";
}
LogNerozpoznanyVstup::LogNerozpoznanyVstup(const QString &v, const QString &ch){
   vstup = v;
   vstup.replace("&","&amp;");
   vstup.replace("<","&lt;");
   vstup.replace(">","&gt;");
   chyboveHlaseni=ch;
}
QString LogNerozpoznanyVstup::echo(){
   return QString("<SPAN CLASS=\"NerozpoznanyPrikaz\">")+vstup+"</SPAN>"+LogRadek::echo();
}
QString LogPrikaz::echo(){
   if (!komentar.isEmpty()) return QString("&nbsp;&nbsp;<SPAN CLASS=\"Komentar\">#")+komentar+"</SPAN>"+LogRadek::echo();
   else return LogRadek::echo();
}

LogPrikazWrite::LogPrikazWrite(){
   typ = LogTypPrikazWrite;
}

LogPrikazWrite::~LogPrikazWrite (){
   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
      delete *tmp;
   }
}

void LogPrikazWrite::pridejParametr(SVyraz * p){
   parametry.push_back(p);
}


QString LogPrikazWrite::echo(){
   QString str=QString("<SPAN CLASS=\"Identifikator\">write</SPAN>");
   str+=QString("<SPAN CLASS=\"ZavorkaFce\">(</SPAN> ");

   bool a = true;

   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
      if (a) a=false;
      else str+=QString(" <SPAN CLASS=\"ZavorkaFce\">,</SPAN> ");
      str+= (*tmp)->echo();
   }
   str+=QString(" <SPAN CLASS=\"ZavorkaFce\">)</SPAN>");
   str+=LogPrikaz::echo();
   if (vystup!="") str += "<BR>\n"+QString("<SPAN CLASS=\"Vystup\">")+vystup+"</SPAN>";
   return str;
}

void LogPrikazWrite::vygenerujVystup(){
   ParametryFce par;
   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
      if ((*tmp)->typ == Retez) {
         // parametr je retezec
         vystup += ((SRetez*)(*tmp))->hodnota;
      }
      else {
         // parametr je cokoliv jineho, takze vyraz
         Prekladac::vnoreni.reset();
         vystup += Stringify((*tmp)->eval(&par),12);
      }
   }
}

LogDefinice::LogDefinice (Definice * d) {
   typ = LogTypDefinice;
   definice = d;
   _jeFce=false;
}

LogDefinice::~LogDefinice(){
   if (vyraz) delete vyraz;
}
QString LogDefinice::echo(){
   QString str=QString("<SPAN CLASS=\"%1\">").arg((jeFce())?("DefiniceFce"):("Definice"));
   if (definice->cisloRadku == cisloRadku && definice->cachePlatna && (!jeFce())) str+=QString("<SPAN CLASS=\"DefinicePlatna\">");
   str+= definice->jmeno;
   if (definice->cisloRadku == cisloRadku && definice->cachePlatna && (!jeFce())) str+="</SPAN>";
   str+="</SPAN>";
   str+=spolecneEcho();
   return str+LogPrikaz::echo();
}

LogPrikazDef::LogPrikazDef(){
   vystup=Prekladac::definice.echo();
}

QString LogPrikazDef::echo(){
   QString str = QString("<SPAN CLASS=\"Identifikator\">def</SPAN>");
   str += LogPrikaz::echo();
   str += "<BR>\n"+QString("<SPAN CLASS=\"Vystup\">")+vystup+"</SPAN>";
   return str;
}

QString LogPrikazDefInt::echo(){
   QString str = QString("<SPAN CLASS=\"Identifikator\">defint</SPAN>");
   str += LogPrikaz::echo();
   str += "<BR>\n"+QString("<SPAN CLASS=\"Vystup\">")+Prekladac::definice.echoInt()+"</SPAN>";
   return str;
}

Log::Log(){
   cisloRadku = 1;
}

void Log::pridej(LogRadek * r) {
   seznam.push_back(r);
   cisloRadku++;
}

void Log::smaz(){
   for (list<LogRadek*>::iterator tmp = seznam.begin(); tmp!=seznam.end(); tmp++) {
      delete *tmp;
   }
   seznam.clear();
   cisloRadku=1;
}

QString Log::echo(){
   QString str = "<html><head><style>";
   str+= Format::styly;
   str +="</style></head><body>";

   str+="<table cellspacing=0 cellpadding=2>";
   int i=0;
   for (list<LogRadek*>::iterator tmp = seznam.begin(); tmp!=seznam.end(); tmp++) {
      i++;
      str += "<tr><td align=right width=30 bgcolor=\"#dddddd\">&nbsp;"+Stringify(i)+":&nbsp;</td><td>";
      str += (*tmp)->echo();
      str += "</td></tr>\n";
   }
   str+="</table></body></html>";
   return str;
}

