/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>
#include <QObject>
#include <qtextcodec.h>
#include "hlavniokno.h"
#include "help.h"
#include "prompt.h"
#include <qfileinfo.h>
#include "format.h"
#include <QTimer>
#include <QSplashScreen>

/**
 * @mainpage Projekt MiniMath
 * Interpreter jednoducheho jazyka matematickych vyrazu.
 * @version verze 1.0
 * @author Martin Plicka <plicka.m@atlas.cz>
 */

/**
 * Ukazatel na instanci hlavniho okna
 */
HlavniOkno * hOkno;

/**
 * Ukazatel na instanci okna vstupu
 */
Prompt * dVstup;

/**
 * Adresar aplikace
 */
QString Adresar;

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   // nastavi kodovani vstupniho textu na UTF-8
   QTextCodec::setCodecForCStrings ( QTextCodec::codecForName("UTF-8") );
   QTextCodec::setCodecForLocale ( QTextCodec::codecForName("UTF-8") );


   HlavniOkno okno;
   hOkno = &okno;

   //Splash * splash = new Splash();
   //splash->show();
   QPixmap splashScreen (":/Splash/images/splash.png");
   QSplashScreen splash (splashScreen);
   splash.setWindowFlags(Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint);
   splash.show();

   Adresar = QFileInfo(app.arguments().at(0)).absolutePath();

   Format::nacti(Adresar+"/style.css");


   Help help(Adresar);
   QObject::connect(okno.actionDokumentace, SIGNAL(activated()), &help, SLOT(zobrazit()));

   Prompt vstup;
   dVstup = & vstup;

   QTimer *timer = new QTimer(&splash);
   QObject::connect(timer, SIGNAL(timeout()), &splash, SLOT(hide()));

   QTimer *timer2 = new QTimer(hOkno);
   QObject::connect(timer2, SIGNAL(timeout()), hOkno, SLOT(show()));

   timer->start(3000);
   //okno.show();
   timer2->start(1000);

   return app.exec();

}
