/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mnozina_definic.h"
#include "zabudovane_fce.h"
#include "definice.h"
#include <set>

using std::pair;
using std::set;

MnozinaDefinic::MnozinaDefinic(){
   init();
}

MnozinaDefinic::~MnozinaDefinic() {
   for (std::set<Definice *, PorovnejDefinice>::iterator tmp=seznam.begin(); tmp!=seznam.end(); tmp++)
      delete *tmp;
}

Definice * MnozinaDefinic::pridej (const QString &co) {
   //nejprve vytvorime tmp na stacku, kdyz nenajdeme tak pridame novy objekt z haldy
   //radeji dvakrat projit strom nez vytvaret pri kazdem dotazu objekt na haldu

   Definice chceme_najit(co);
   set<Definice *, PorovnejDefinice>::iterator tmp;
   tmp = seznam.find(&chceme_najit);
   if (tmp!= seznam.end()) return *tmp;
   else {
      pair<set<Definice *, PorovnejDefinice>::iterator, bool> vysledek;
      vysledek = seznam.insert(new Definice(co));
      return *(vysledek.first);
   }
}

void MnozinaDefinic::smaz(){
   seznam.clear();
   init();
}

void MnozinaDefinic::init(){
   seznam.insert(new DefiniceZabudovanaSin());
   seznam.insert(new DefiniceZabudovanaPi());
   seznam.insert(new DefiniceZabudovanaASin());
   seznam.insert(new DefiniceZabudovanaSqrt());
}

int MnozinaDefinic::pocetPrvku () {
   return (int)seznam.size();
}

QString MnozinaDefinic::echo(){
   QString str="... Použité identifikátory: ...<BR>\n";
   for (set<Definice *, PorovnejDefinice>::iterator tmp=seznam.begin(); tmp!=seznam.end(); tmp++)
      if (!(*tmp)->zabudovana) str+= "&nbsp;&nbsp;"+(*tmp)->echo() +"<BR>";
   return str;
}

QString MnozinaDefinic::echoInt(){
   QString str="... Zabudované funkce a konstanty ...<BR>\n";
   for (set<Definice *, PorovnejDefinice>::iterator tmp=seznam.begin(); tmp!=seznam.end(); tmp++)
      if ((*tmp)->zabudovana) str+= "&nbsp;&nbsp;"+(*tmp)->echo() +"<BR>";
   return str;
}

