/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MNOZINA_DEFINIC_H_INCLUDED
#define MNOZINA_DEFINIC_H_INCLUDED

#include "porovnej_definice.h"
#include <set>
#include "qstring.h"

class Definice;

/**
 * Mnozina, ktera uchovava definice jednotlivych promennych/funkci
 * @author Martin Plicka
 */
class MnozinaDefinic {
      /* Bohuzel musime uchovavat pouze mnozinu ukazatelu na definice,
      set by mohla byt implementovana tak, ze by presouvala prvky v pameti,
      coz nechceme, ztratili bychom ukazatele z jinych casti programu*/

      // mnozina definic z STL
      std::set<Definice*, PorovnejDefinice> seznam;

   public:

      MnozinaDefinic();

      ~MnozinaDefinic();

      /**
       * prida definici do mnoziny, pokud tam neni.
       * @param co retez se jmenem definice
       * @return ukazatel na definici
       */
      Definice * pridej (const QString &co);

      /**
       * Vymaze veskere definice
       */
      void smaz();

      /**
       * Inicializuje mnozinu definic na preddefinovane
       */
      void init();

      /**
       * pocet jiz vlozenych definic
       * @return pocet vlozenych definic
       */
      int pocetPrvku ();

      /**
       * Vypise definovane funkce
       * @return text vypisu
       */
      QString echo();

      /**
       * Vypise interni funkce
       * @return text vypisu
       */
      QString echoInt(); //vypise interni funkce
};


#endif // MNOZINA_DEFINIC_H_INCLUDED
