/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "prekladac.h"
#include "hlavniokno.h"
#include "lexan.h"
#include "strtools.h"

#include "definice.h"
#include "syntaxstrom.h"
#include "log.h"
#include "synan.h"
#include "vyjimka.h"


#include <qmessagebox.h>

extern HlavniOkno * hOkno;

namespace Prekladac {

using namespace SynAn;

Log log;
MnozinaDefinic definice;
CitacVnoreni vnoreni(500);

void preloz(const QString & str){
   LexAn::init(str);

   LogRadek * radek = 0;
   try {
      radek = Prikaz();
      log.pridej(radek);

   }
   catch (VyjimkaText v) {
      log.pridej(new LogNerozpoznanyVstup(str, "CHYBA: "+v.text));
      goto end;
   }
   catch (VyjimkaReset v) {
      reset();
      goto end;
   }

   // pokud nam synAn vrati definici, provedeme testy
   if (radek->typ == LogTypDefinice) {

      bool presIf;

      //ukazatel na novou definici
      LogDefinice * kandidatDefinice = (LogDefinice*)radek;

      //ukazatel na definici v tabulce
      Definice * puvodniDefinice = kandidatDefinice->definice;


      if (puvodniDefinice->zabudovana) {
         kandidatDefinice->chyboveHlaseni="Zabudovanou funkci nelze předefinovat";
         goto bez_aplikace;
      }


      //vypocti tabulky zavislosti
      kandidatDefinice->zjistiZavislosti();


      //priprava pro detekci rekurze pres if
      presIf = false;

      try {
         // QMessageBox::information(hOkno, "", "Zkousim rekurzi");
         presIf = (kandidatDefinice->zkontrolujRekurzi(puvodniDefinice,false));
      }
      catch (Vyjimka v) {
         kandidatDefinice->chyboveHlaseni="CHYBA: Definice vede na přímou rekurzi bez podmínky, definice není aplikována";
         goto bez_aplikace;
      }

      //varovne hlaseni pri rekurzi pres if
      if (presIf) kandidatDefinice->chyboveHlaseni="Varování: Definice vede na podmíněnou rekurzi";

      //odhlas stare zavislosti
      puvodniDefinice->odhlasZavislosti();

      //zkopirujeme definici na spravne misto
      puvodniDefinice->nastav(*kandidatDefinice);
      //smazeme primou zavislost samu na sobe, uz neni treba, v budoucnu by zpusobila zacykleni
      //rekurze pres if muze zustat, detekce pri zjisteni rekurze pres if nepokracuje hloubeji
      puvodniDefinice->smazZavislost(puvodniDefinice);


      //nahlas nove zavislosti
      puvodniDefinice->nahlasZavislosti();
      puvodniDefinice->zneplatni();


bez_aplikace:

      // muzeme smazat seznam zavislosti z logu, ten jej nepotrebuje
      kandidatDefinice->smazZavislosti();
   }
   else if (radek->typ == LogTypPrikazWrite) {
      try {
         ((LogPrikazWrite*)radek)->vygenerujVystup();
      }
      catch (VyjimkaTextRadek v){
         radek->chyboveHlaseni = "CHYBA: "+ v.text +", řádek "+Stringify(v.radek);
      }
      catch (VyjimkaText v){
         radek->chyboveHlaseni = "CHYBA: "+ v.text;
      }
      catch (Vyjimka v) {
         radek->chyboveHlaseni = "Neznámá chyba";
      }
   }
end: ;// uz nic netreba
}

void reset(){

   log.smaz();
   definice.smaz();
}

}//namespace Prekladac
