/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "synan.h"
#include "lexan.h"
#include "vyjimka.h"
#include "prekladac.h"


namespace SynAn {

/**
 * Pomocny ukazatel na prozatim vytvoreny identifikator
 */
LogDefinice * pomocnyIdent;

using namespace LexAn;

SVyraz * Vyraz();

/**
 * Neterminal oznacujici identifikator
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Ident(){
   SVyraz * tmp = 0;
   try {
      srovnej (LexIdentifikator);
      int i;
      if (pomocnyIdent != 0 && (i= pomocnyIdent->indexParametru(lexRetez))>-1) {
         //je to lokalni parametr definovane funkce
         return new SLokalniPromenna(i, pomocnyIdent);
      }
      else {
         //je to volani fce, promenne
         Definice * helper;
         tmp = new SIdentifikator(helper = Prekladac::definice.pridej(lexRetez));
         if (ukaz()==LexLZavorka) {
            ((SIdentifikator*)tmp)->jeFce(true);
            srovnej(LexLZavorka);
            if (ukaz()!=LexPZavorka){
               ((SIdentifikator*)tmp)->pridejParametr(Vyraz());
               while (ukaz()==LexCarka) {
                  srovnej(LexCarka);
                  ((SIdentifikator*)tmp)->pridejParametr(Vyraz());
               }
            }
            srovnej(LexPZavorka);
         }
         // zde overime, jestli pri rekurzivnim volani odpovidaji pocty parametru pri volani a definici
         if (pomocnyIdent != 0 && (((SIdentifikator*)tmp)->definice == pomocnyIdent->definice)) {
            if (((SIdentifikator*)tmp)->jeFce() != pomocnyIdent->jeFce())
               throw VyjimkaText(QString("Proměnná/Funkce <b>")+pomocnyIdent->definice->jmeno+"</b> volá sama sebe se špatnými parametry");
            if (((SIdentifikator*)tmp)->pocetParametru() > pomocnyIdent->pocetParametru())
               throw VyjimkaText(QString("Funkce <b>")+pomocnyIdent->definice->jmeno+"</b> volá sama sebe s příliš vysokým počtem parametrů");
         }
         return tmp;
      }
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
}

/**
 * Nejspodnejsi element = cislo, if, read nebo zavorky
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Prvek2(){
   SVyraz * tmp = 0;
   try{
      switch (ukaz()){
      case LexIf:
         srovnej(LexIf);
         srovnej(LexLZavorka);
         tmp = new SIf(Vyraz());
         srovnej(LexCarka);
         ((SIf*)tmp)->kdyzPravda = Vyraz();
         srovnej(LexCarka);
         ((SIf*)tmp)->kdyzNepravda = Vyraz();
         srovnej(LexPZavorka);
         break;
      case LexRead:
         srovnej(LexRead);
         srovnej(LexLZavorka);
         srovnej(LexRetez);
         tmp = new SRead(new SRetez(lexRetez));
         srovnej(LexPZavorka);
         break;
      case LexIdentifikator:
         tmp = Ident();
         break;
      case LexLZavorka:
         srovnej(LexLZavorka);
         tmp=new SZavorky(Vyraz());
         srovnej(LexPZavorka);
         break;
      default:
         srovnej(LexCislo);
         return new SCislo(lexHodnota);
         break;
      }
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
   return tmp;
}

/**
 * Pripadna negace znaminkem -
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Prvek () {
   switch (ukaz()) {
   case LexMinus:
      srovnej(LexMinus);
      return new SUnMinus(Prvek2());
      break;
   case LexNot:
      srovnej(LexNot);
      return new SNot(Prvek2());
      break;
   }
   return Prvek2();
}


/**
 * Patro pro x-te mocniny
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Mocnina(){
   SVyraz * tmp=0;
   try {
      tmp = Prvek();
      if (ukaz()==LexMocnina) {
         srovnej(LexMocnina);
         return new SMocnina(tmp,Prvek());
      }
      return tmp;
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
}

/**
 * Patro pro nasobeni a deleni
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Aritm2(){
   SVyraz * tmp=0;
   try {
      tmp = Mocnina();
      while (ukaz()==LexKrat || ukaz()==LexDeleno){
         if (ukaz()==LexKrat) {
            srovnej(LexKrat);
            tmp = new SKrat(tmp,Mocnina());
         }
         else {
            srovnej(LexDeleno);
            tmp = new SDeleno(tmp,Mocnina());
         }
      }
      return tmp;
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
}

/**
 * Patro pro scitani a odcitani
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Aritm1(){
   SVyraz * tmp=0;
   try {
      tmp = Aritm2();
      while (ukaz()==LexPlus || ukaz()==LexMinus){
         if (ukaz()==LexPlus) {
            srovnej(LexPlus);
            tmp = new SPlus(tmp,Aritm2());
         }
         else {
            srovnej(LexMinus);
            tmp = new SMinus(tmp,Aritm2());
         }
      }
      return tmp;
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
}


/**
 * Patro pro relacni operatory porovnani
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Relacni(){
   SVyraz * tmp=0;
   try {
      tmp = Aritm1();
      switch (ukaz()){
      case LexVetsi:
         srovnej(LexVetsi);
         return new SVetsi(tmp,Aritm1());
         break;
      case LexMensi:
         srovnej(LexMensi);
         return new SMensi(tmp,Aritm1());
         break;
      case LexVetsiRovno:
         srovnej(LexVetsiRovno);
         return new SVetsiRovno(tmp,Aritm1());
         break;
      case LexMensiRovno:
         srovnej(LexMensiRovno);
         return new SMensiRovno(tmp,Aritm1());
         break;
      case LexRovno:
         srovnej(LexRovno);
         return new SRovno(tmp,Aritm1());
         break;
      case LexNerovno:
         srovnej(LexNerovno);
         return new SNerovno(tmp,Aritm1());
         break;
      }
      return tmp;
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
}

/**
 * Patro pro cely vyraz. Nahore je log. AND, OR
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Vyraz(){
   SVyraz * tmp=0;
   try {
      tmp = Relacni();
      while (ukaz()==LexAnd || ukaz()==LexOr){
         if (ukaz()==LexAnd) {
            srovnej(LexAnd);
            tmp = new SAnd(tmp,Relacni());
         }
         else {
            srovnej(LexOr);
            tmp = new SOr(tmp,Relacni());
         }
      }
      return tmp;
   }
   catch (...){
      if (tmp) delete tmp;
      throw;
   }
}

/**
 * Parametr prikazu write, cislo nebo retezec
 * @return ukazatel na vrchol prelozeneho stromu
 */
SVyraz * Writeparam(){
   if (ukaz()==LexRetez) {
      srovnej(LexRetez);
      return new SRetez(lexRetez);
   }
   else{
      return Vyraz();
   }
}


LogRadek * Prikaz (){
   pomocnyIdent=0;
   LogRadek * tmp=0;
   try{
      switch (ukaz()){
      case LexIdentifikator:
         srovnej(LexIdentifikator);
         tmp = new LogDefinice(Prekladac::definice.pridej(lexRetez));
         pomocnyIdent = (LogDefinice*)tmp;
         if (ukaz()==LexLZavorka) {
            pomocnyIdent->jeFce(true);
            srovnej(LexLZavorka);
            if (ukaz()==LexIdentifikator){
               if (lexRetez==(pomocnyIdent->definice->jmeno)) throw VyjimkaText ("Parametry musí být různé jméno od jména funkce");
               pomocnyIdent->pridejParametr(lexRetez);
               srovnej(LexIdentifikator);
               while (ukaz()==LexCarka) {
                  srovnej(LexCarka);
                  srovnej(LexIdentifikator);
                  if (lexRetez==(pomocnyIdent->definice->jmeno)) throw VyjimkaText ("Parametry musí mít různé jméno od jména funkce");
                  if (!pomocnyIdent->pridejParametr(lexRetez)) throw VyjimkaText ("Parametry musí mít různá jména");
               }
            }
            srovnej(LexPZavorka);
         }
         srovnej(LexPrirazeni);
         pomocnyIdent->vyraz = Vyraz();
         break;
      case LexWrite:
         srovnej(LexWrite);
         tmp = new LogPrikazWrite();
         srovnej(LexLZavorka);
         ((LogPrikazWrite*)tmp)->pridejParametr(Writeparam());
         while (ukaz()==LexCarka){
            srovnej(LexCarka);
            ((LogPrikazWrite*)tmp)->pridejParametr(Writeparam());
         }
         srovnej(LexPZavorka);
         break;
      case LexEnter:
         tmp = new LogPrikaz();
         break;
      case LexDef:
         srovnej(LexDef);
         tmp = new LogPrikazDef();
         break;
      case LexDefInt:
         srovnej(LexDefInt);
         tmp = new LogPrikazDefInt();
         break;
      case LexReset:
         srovnej (LexReset);
         srovnej (LexEnter);
         throw VyjimkaReset();
         break;
      default:

         //kdyby nahodou
         throw VyjimkaText ("Nerozpoznaný příkaz");
         break;
      }
      srovnej(LexEnter);
   }
   catch (...) {
      if (tmp) delete tmp;
      throw;
   }
   ((LogPrikaz*)tmp)->komentar=lexRetez;
   return tmp;
}


} // namespace SynAn
