/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "zabudovane_fce.h"
#include <math.h>
#include <vector>

DefiniceZabudovana::DefiniceZabudovana(const QString &jmeno): Definice(jmeno){
   zabudovana = true;
}

QString DefiniceZabudovana::echo(){
   QString str=QString("<SPAN CLASS=\"%1\">").arg((jeFce())?("DefiniceFce"):("Definice"));
   str+= jmeno+"</SPAN>";
   if (jeFce()){
      str+="<SPAN CLASS=\"ZavorkaDefinice\">(</SPAN>";
      bool a = true;
      for (std::vector<QString>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
         if (a) a=false;
         else str+=",";
         str+= QString("<SPAN CLASS=\"DefiniceParametr\">")+(*tmp)+"</SPAN>";
      }
      str+=QString("<SPAN CLASS=\"ZavorkaDefinice\">)</SPAN>");
   }
   str+=" - " + popis();
   return str;
}


/** SIN **/

DefiniceZabudovanaSin::DefiniceZabudovanaSin(): DefiniceZabudovana("sin"){
   pridejParametr("x");
}

QString DefiniceZabudovanaSin::popis(){
   return "vrací hodnotu funkce sinus, x je v radiánech";
}

double DefiniceZabudovanaSin::eval(ParametryFce * par){
   return sin((*par)[0]);
}

/** ArcSIN **/

DefiniceZabudovanaASin::DefiniceZabudovanaASin(): DefiniceZabudovana("asin"){
   pridejParametr("x");
}

QString DefiniceZabudovanaASin::popis(){
   return "vrací hodnotu funkce arcussinus v radiánech";
}

double DefiniceZabudovanaASin::eval(ParametryFce * par){
   return asin((*par)[0]);
}

/** PI **/

DefiniceZabudovanaPi::DefiniceZabudovanaPi() : DefiniceZabudovana("pi"){
}

QString DefiniceZabudovanaPi::popis(){
   return "Ludolfovo číslo";
}

double DefiniceZabudovanaPi::eval(ParametryFce *par){
   return M_PI;
}

/** SQRT **/

DefiniceZabudovanaSqrt::DefiniceZabudovanaSqrt(): DefiniceZabudovana("sqrt"){
   pridejParametr("x");
}

QString DefiniceZabudovanaSqrt::popis(){
   return "druhá odmocnina x";
}

double DefiniceZabudovanaSqrt::eval(ParametryFce * par){
   return sqrt((*par)[0]);
}
