/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZABUDOVANE_FCE_H_INCLUDED
#define ZABUDOVANE_FCE_H_INCLUDED

#include "definice.h"

/**
 * Abstraktni trida pro vytvareni zabudovanych funkci.
 * Pri vytvareni novych zabudovanych funkci je nutno definovat metody echo(), popis() a konstruktor,
 * vcetne vytvoreni parametru dane funkce.
 * @author Martin Plicka
 */
class DefiniceZabudovana: public Definice{
   public:
      /**
       * Konstruktor.
       * @brief Konstruktor.
       * @param jmeno Jmeno zabudovane funkce
       */
      DefiniceZabudovana(const QString &jmeno);

      virtual bool jeFce(){
         return pocetParametru();
      };
      /**
       * Popisuje chovani zabudovane funkce. Vypise se v programu po zadani prikazu "defint"
       * @return retezec s popisem chovani funkce
       */
      virtual QString popis() = 0; //musi byt predefinovana v podtride

      /**
       * Funkce definuje HTML formatovany vystup definice do logu programu
       * @return Textova reprezentace definice
       */
      virtual QString echo ();

      /**
       * Vyhodnocuje funkci
       * @param parametry vypoctene hodnoty parametru predavanych pri vypoctu funkci
       * @return vysledek
       */
      virtual double eval(ParametryFce * parametry)=0; //musi byt predefinovana v podtride
};

/**
 * Zabudovana Funkce Sinus
 * @author Martin Plicka
 */
class DefiniceZabudovanaSin: public DefiniceZabudovana{
   public:
      DefiniceZabudovanaSin();
      virtual QString popis();
      virtual double eval(ParametryFce*);
};

/**
 * Zabudovana funkce Arcus Sinus
 * @author Martin Plicka
 */
class DefiniceZabudovanaASin: public DefiniceZabudovana{
   public:
      DefiniceZabudovanaASin();
      virtual QString popis();
      virtual double eval(ParametryFce*);
};

/**
 * Zabudovana konstanta PI
 * @author Martin Plicka
 */
class DefiniceZabudovanaPi: public DefiniceZabudovana{
   public:
      DefiniceZabudovanaPi();
      virtual QString popis();
      virtual double eval(ParametryFce*);
};

/**
 * Zabudovana funkce Odmocnina
 * @author Martin Plicka
 */
class DefiniceZabudovanaSqrt: public DefiniceZabudovana{
   public:
      DefiniceZabudovanaSqrt();
      virtual QString popis();
      virtual double eval(ParametryFce*);
};

#endif // ZABUDOVANE_FCE_H_INCLUDED
